# hyperidentity

![stability-experimental](https://img.shields.io/badge/stability-experimental-orange.svg?style=flat-square)
[![NPM Version](https://img.shields.io/npm/v/hyperidentity.svg)](https://www.npmjs.com/package/hyperidentity)
[![JavaScript Style Guide](https://img.shields.io/badge/code%20style-standard-brightgreen.svg)](http://standardjs.com/)

A decentralized authentication and data-sharing protocol. Hyperidentity allows you to

* **Own your data**:  Provide your data to a web services but still controll what you've shared.
* **Avoid vendor lock-in**: Move your data between services.
* **No single-point-of-failure**: Every service can have a up-to-date copy of your data. There won't be a single service depended by all other services. Think github and all other Code Review/Deploy/Project Management tools.

and many other cool things!

`npm i -g hyperidentity`

## Usage

* Create an identity at `./my-id`:

```
$ hi init my-id
56d0a72488190d37aaa28447a5600eafe67df00bf89ab646def449c17e331a56 // your identity key
```

* Login to a service:

```
$ hi login my-id token
```

* bring your identity online:

```
$ hi up my-id
```

## API

check out `test/signup.js`, `test/service.js`, and `example` to see how to implement a web service accepting hyperidentity.

### Identity

An identity is a user with the data he/she want to share with the service. It can also replicate data generated by web services.

`var identity = require('hyperidentity').identity`

#### `id = identity(archive)`

Create a new identity with a [hyperdrive](https://github.com/mafintosh/hyperdrive) archive.

#### `id.archive`

The archive used by this ID.

#### `id.setMeta(meta, cb)`

Set metadata of the ID

#### `id.getMeta(cb)`

Get metadata of the ID

#### `token = id.serviceLinkToken(service, archiveKey)`

Create a link token for `id` for the following purpose:

1. verify user really own the ID(archive)
2. give user a service-owned archive to [link](https://github.com/poga/hyperdrive-ln) to its ID.

`service` is an object with two key: `{publicKey, secretKey}`. You need to use `sodium-signatures` to create a keyPair: `signatures.keyPair()`

Returns a token string.

#### `id.acceptLinkToken(token, cb(err))`

Accept a link token. Under the hood, this will:

1. write a response to `.proofs/${service.publicKey}`
2. link `archiveKey` at `.links/${service.publicKey}

#### `id.verifyAcceptingness(service, cb(err, verified))`

Check whether `id` accepted the link token from `service`.

### Service

A service is a web service that allows users to login with their hyperidentity a.k.a hyperservice.

`var service = require('hyperidentity').service`

#### `var s = service(drive, meta, storagePath, [keyPair])`

Create a new hyperservice.

Identity data will be stored in `storagePath`.

If `keyPair` is empty, it will generate one for you.

#### `s.createIdentity(key)`

Create a hyperidentity from given key.

#### `s.issue(identity)`

Issue a **link token** for that identity.

#### `s.verify(identity, cb)`

Verify whether identity is accepted the link token. This method will block if identity is not online.

## License

The MIT License
